@@ -22,6 +22,7 @@
 #include "SystemFonts.h"
 #include "IconColors.h"
 #include "UnityLauncher.h"
+#include "FeaturedModel.h"
 
 // Qt includes
 #include <QAction>
@@ -95,6 +96,7 @@ DiscoverMainWindow::DiscoverMainWindow(CompactMode mode)
     qmlRegisterType<PaginateModel>("org.kde.discover.app", 1, 0, "PaginateModel");
     qmlRegisterType<IconColors>("org.kde.discover.app", 1, 0, "IconColors");
     qmlRegisterType<KConcatenateRowsProxyModel>("org.kde.discover.app", 1, 0, "KConcatenateRowsProxyModel");
+    qmlRegisterType<FeaturedModel>("org.kde.discover.app", 1, 0, "FeaturedModel");
     qmlRegisterType<QSortFilterProxyModel>("org.kde.discover.app", 1, 0, "QSortFilterProxyModel");
 
     qmlRegisterSingletonType<SystemFonts>("org.kde.discover.app", 1, 0, "SystemFonts", ([](QQmlEngine*, QJSEngine*) -> QObject* { return new SystemFonts; }));
@@ -183,6 +185,11 @@ void DiscoverMainWindow::openCategory(const QString& category)
 
 void DiscoverMainWindow::openLocalPackage(const QUrl& localfile)
 {
+    if (!QFile::exists(localfile.toLocalFile())) {
+//         showPassiveNotification(i18n("Trying to open unexisting file '%1'", localfile.toString()));
+        qWarning() << "Trying to open unexisting file" << localfile;
+        return;
+    }
     rootObject()->setProperty("defaultStartup", false);
     auto action = new OneTimeAction(
         [this, localfile]() {
